# Copyright (C) 2018 Fassio Blatter
version="TooOld"
import stopeight.legacy
from stopeight.util.editor.data import ScribbleData

import stopeight.logging as log

def stroke_parallel(data:ScribbleData)->ScribbleData:
    return _convert(stopeight.legacy.stroke_parallel(list(map(tuple,data['coords'].tolist()))))

def stroke_sequential(data:ScribbleData)->ScribbleData:
    # type: (ScribbleData) -> ScribbleData
    return _convert(stopeight.legacy.stroke_sequential(list(map(tuple,data['coords'].tolist()))))

def _parse_file(filename):
    legacy_data= stopeight.legacy.parse_file(filename)
    return _convert(legacy_data)

def _convert(legacy_data):
    scribbledata = ScribbleData(size=len(legacy_data))
    for i,v in enumerate(legacy_data):
        scribbledata[i]['coords'] = [v[0],v[1]]
    return scribbledata

def open_SP()->ScribbleData:
    from stopeight.util.editor.runnable import EditorApp
    from PySide2.QtWidgets import QFileDialog
    filename = QFileDialog.getOpenFileName(EditorApp().window)
    assert type(filename[0]) == str
    assert len(filename[0]) != 0
    data = _parse_file(filename[0])
    return data

def TCT_to_bezier(data:ScribbleData)->ScribbleData:
    return stopeight.legacy.TCT_to_bezier(data)

import stopeight.finders
from stopeight.getters import ListSwitchable

def _intList_to_ScribbleData(original: ScribbleData, mylist: list)->ScribbleData:
    assert len(original)>1
    newsize=len(mylist)+2
    data = ScribbleData(size=newsize)
    data[0]['coords']=[original[0]['coords'][0],original[0]['coords'][1]]
    data[newsize-1]['coords']=[original[-1]['coords'][0],original[-1]['coords'][1]]
    for i,index in enumerate(mylist):
        data[i+1]['coords'] = [original[index]['coords'][0],original[index]['coords'][1]]
    return data

def sequential(data:ScribbleData)->ScribbleData:
    log.info("Invoking findCliffs...")
    ql = stopeight.getters.ListSwitchable(data)
    turns = stopeight.finders.findCliffs(ql)
    print(turns)
    assert type(turns) == list
    return _intList_to_ScribbleData(data,turns)

def parallel(data:ScribbleData)->ScribbleData:
    log.info("Invoking findSpiralCliffs...")
    ql = stopeight.getters.ListSwitchable(data)
    turns = stopeight.finders.findSpiralCliffs(ql)
    print(turns)
    assert type(turns) == list
    return _intList_to_ScribbleData(data,turns)

import stopeight.getters
import numpy as np

def getFirstTurnByTriplets(data:ScribbleData)->ScribbleData:
    log.setLevel(log.DEBUG)
    log.info("Invoking getFirstTurnByTriplets with "+str(len(data))+" data points...")
    ta = stopeight.getters.TurnAnalyzer(data)
    log.debug("Invoking turn chopping algorithm length "+str(len(ta)))
    out = ta.getFirstTurnByTriplets(ta)
    log.debug("Read length of a object of type "+str(type(out))+" and length "+str(len(out)))
    return out.view(ScribbleData)

def getFirstCorner(data:ScribbleData)->ScribbleData:
    log.setLevel(log.DEBUG)
    log.info("Invoking getFirstCorner with "+str(len(data))+" data points...")
    ql = stopeight.getters.ListSwitchable(data)
    log.debug("Invoking corner chopping algorithm length "+str(len(ql)))
    out = stopeight.getters.getFirstCorner(ql)
    log.debug("Read length of a object of type "+str(type(out))+" and length "+str(len(out)))
    return out.view(ScribbleData)



def fileAdapter(filename):
    import wave
    from stopeight.util.editor.modules import file_wave
    spf = wave.open(filename,'r')
    if spf.getcomptype()!='NONE':
        raise Exception("Compressed WAV files are not supported.")
    total_channels = spf.getnchannels()
    allChannels=[]
    for i in range(total_channels):
        allChannels.append(file_wave._getChannel(spf,i,total_channels))
    return allChannels

def allGraphs(allChannels):
    from stopeight.util.editor.modules import grapher
    import time
    graphs=[]
    a = time.time()
    for channel in allChannels:
        graphs.append(grapher._append(grapher.create_vector_graph(channel)))
    b = time.time()
    print("Elapsed: "+str(b-a))
    return graphs


def test_NonAveraged():
    from stopeight.util.parser import process_directory
    graphs = process_directory('./','.wav',fileAdapter,allGraphs)


def test_linePrinter():
    from stopeight.util.TCT_printer import TCTPrinter
    from stopeight import legacy
    import os
    myprinter = TCTPrinter('linePrinter.out')
    fileline = legacy.parse_file(os.path.join('stopeight-clibs','legacy','test-data','v0-1-0','stage3-build04-authWorking','almostStraightLeg.sp'))
    myprinter.lines(fileline)
    fileline = legacy.stroke_sequential(fileline)
    fileline = legacy.TCT_to_bezier(fileline)
    myprinter.TCTs(fileline)
    myprinter.write()

def test_tableauPrinter():
    from stopeight.util.editor.data import ScribbleData
    from stopeight.util.tableau_printer import tPrinter
    import numpy
    printer = tPrinter('tableauPrinter.out',3)
    data = ( (0,0),(0,256),(0,256),(256,256),(256,256),(256,0),(256,0),(0,0) )
    nline = ScribbleData(size=len(data))
    for i,v in enumerate(data):
        nline[i]['coords'] = [v[0],v[1]]
    printer.draw(nline)
    print('testing offset function '+str(printer.offset(nline[0])))
    printer.draw(nline)
    print('testing offset function '+str(printer.offset(nline[0])))
    printer.draw(nline)
    printer.write()

def test_TCTPrinter():
    from stopeight.util.TCT_printer import TCTPrinter
    import numpy
    printer = TCTPrinter('TCTPrinter.out')
    nline = numpy.array(((200,200),(200,300)))
    printer.lines(nline)
    printer.lines(nline)
    printer.lines(nline)
    #printer.scene.add(SVG.Line((200,200),(200,300)))
    printer.write()



def test_UserFiles():
    from stopeight.util import editor
    from stopeight.util.editor.modules import legacy
    from stopeight.util.editor.modules import file
    from stopeight.util.parser import process_directory
    from pathlib import Path
    lines = process_directory('.stopeight','.npy',file._read,legacy.stroke_sequential,base_dir=Path.home(),filename='UserFiles.out',drawResize=True)


from numpy import ndarray
import numpy
#todo make static call
#no module dependency here, so copy of print(array_instance.dtype)
Vectors_dtype = numpy.dtype({'names':['coords','type','tct_type','cov_type'], 'formats':[('<f8', (2,)),'<i4','<i4','<i4'], 'offsets':[0,16,20,24], 'itemsize':32})

class ScribbleData(ndarray):
    def __new__(cls,*args,**kwargs):
        try:
            if kwargs['size']>1:
                kwargs['shape']=(kwargs['size'])
            else:
                kwargs['shape']=(1)
            del kwargs['size'] 
        except KeyError:
            kwargs['shape']=(1)
#todo enums
        kwargs['dtype']= numpy.dtype(Vectors_dtype)
        return super(ScribbleData,cls).__new__(cls,*args,**kwargs)

class WaveData(ndarray):
    pass



#!/usr/bin/env python

# Copyright (C) 2009-2016 Specific Purpose Software GmbH

from stopeight.util.editor.data import ScribbleData
from stopeight.util.editor.scribble import ScribbleArea

import stopeight.logging as log
log.disable(log.CRITICAL)

def save_NPY(data):
    # this is not a real error but saves us from writing more code
    data=data.data
    log.error("DISREGARD THIS ERROR, IT HAS BEEN TRIGGERED TO SAVE THE FILE")
    raise ValueError
save_NPY.__annotations__ = {'data': ScribbleArea, 'return': type(None)}

def save_as_NPY(data):
    from stopeight.util.editor.runnable import EditorApp
    from PySide2.QtWidgets import QFileDialog
    filename = QFileDialog.getSaveFileName(EditorApp().window,"QFileDialog.getSaveFileName()","","NumPy Files (*.npy)")
    assert type(filename[0]) == str
    assert len(filename[0]) != 0
    _to_file(filename[0],data.data)
    log.info("Saved to File "+str(filename[0]))
    return filename
save_as_NPY.__annotations__ = {'data': ScribbleArea, 'return': type(None)}

def open_NPY():
    from stopeight.util.editor.runnable import EditorApp
    from PySide2.QtWidgets import QFileDialog
    filename = QFileDialog.getOpenFileName(EditorApp().window)
    assert type(filename[0]) == str
    assert len(filename[0]) != 0, "No filename"
    log.debug("Reading from File "+str(filename[0]))
    return _read(filename[0])
open_NPY.__annotations__ = {'return': ScribbleData}

def _read(path):
    if path.endswith('.npy'):
        import numpy
        arr = numpy.load(path)
        if arr.dtype==ScribbleData().dtype:
            arr=arr.view(ScribbleData)
        return arr
    elif path.endswith('.json'):
        import json
        f = open(path,'r')
        return json.load(f)

def _write(data,timestamp,outdir=None):
    if outdir==None:
        from os.path import expanduser,join
        outdir=join(expanduser("~"))
    import datetime
    time = (datetime.datetime.utcfromtimestamp(timestamp))
    time = time.strftime('%Y%m%dT%H%M%S')
    name = time+"."+type(data).__name__
    import os
    from os import path
    if not os.path.exists(outdir):
        os.makedirs(outdir)
    path = path.join(outdir,name)
    log.info("Writing method input data to: "+path)
    _to_file(path,data)

def _to_file(path,data):
    #import pickle
    #picl = pickle.Pickler(open(path+'.pickle','wb'),pickle.HIGHEST_PROTOCOL)
    #picl.dump(_DATA['MyScribble'].INPUT)

    import numpy
    if type(data).__bases__[0] == numpy.ndarray:
        numpy.save(path,data,allow_pickle=True,fix_imports=True)#python3: fix_imports=False
    else:
        import json
        f = open(path+'.json','w')
        json.dump(data,f)



#!/usr/bin/env python

# Copyright (C) 2009-2012 Specific Purpose Software GmbH

from stopeight.comparator.matrixTools import *
from stopeight.comparator import vectorTools
from numpy import zeros_like

import stopeight.logging as log
log.disable(log.CRITICAL)

class LineMatchClass:
    minTolerance = 2

    @staticmethod
    def align(point1,point2,reference,comparison):

        log.info('Moving reference to comparePoint')
        movement = idmatrix()
        movement = dot(translate(-reference[point1]),movement)
        compareRecord = zeros_like(reference)
        for pos,moveme in enumerate(reference):
            compareRecord[pos] = transform(moveme,movement)
        log.debug(compareRecord)
        log.info('We are aligning point %s ' %compareRecord[point2] + 'with %s' %comparison[1])
        referenceAngle = (compareRecord[point2]).getAngleToNegativeXAxis()
        comparisonAngle = (comparison[1]).getAngleToNegativeXAxis()

        angle = referenceAngle - comparisonAngle
        log.info('Rotating comparison: %s' %angle)
        movement = idmatrix()
        movement = dot(rotate(angle),movement)
        match = zeros_like(comparison)
        for pos,moveme in enumerate(comparison):
            match[pos]= transform(moveme,movement)

        log.info('Moving match to destination: %s' %reference[point1])
        movement = idmatrix()
        movement = dot(translate(reference[point1]),movement)
        for pos,moveme in enumerate(match):
            match[pos]= transform(moveme,movement)
        return match

    @staticmethod
    def __getSmallestDistance(line):
        smallestDistance = vectorTools.getVectorLength(line[1]-line[0])
        for i,point in enumerate(line[1:-1]):
            length = vectorTools.getVectorLength(line[i+1]-line[i])
            if length<smallestDistance:
                smallestDistance = length
        return smallestDistance

    @staticmethod
    def __withinToleranceRange(tolerance,point1,point2):
        if point2[0]<point1[0]+tolerance and point2[0]>point1[0]-tolerance and point2[1]<point1[1]+tolerance and point2[1]>point1[1]-tolerance:
            return True
        return False

    @staticmethod
    def compare(tolerance,first,second):
        for i,point in enumerate(first):
            if not LineMatchClass.__withinToleranceRange(tolerance,point,second[i]):
                return False
        return True

    @staticmethod
    def __scaleToAvg(tobescaled,target):
        log.debug('Comparing lengths of vectors in target...')
        dbAvg = target[0:len(tobescaled)].getAverageLength()
        log.debug(dbAvg)
        log.debug('...with lengths of vectors in tobescaled...')
        tobescaledAvg = tobescaled.getAverageLength()
        log.debug(tobescaledAvg)
        movement = idmatrix()
        # Uniform scale:
        scaleFactor = [dbAvg/tobescaledAvg,dbAvg/tobescaledAvg]
        log.debug(scaleFactor)
        log.info('Scale tobescaled to target: %s' %scaleFactor)
        movement = dot(scale(scaleFactor),movement)
        for pos,moveme in enumerate(tobescaled):
            tobescaled[pos]= transform(moveme,movement)
        return tobescaled

    @staticmethod
    def validate_data(invec,compvec):
        # prepare data, doing a check
        if len(invec)<=2 or len(compvec)<=2:
            #Should be replaced by try/except -> throw
            raise Exception('length of input or reference is below 3. Scale-Matching lines of size <3 is not possible.')
            return False
        if len(invec)!=len(compvec):
            raise Exception('length of input is not equal to length of reference')
            return False
        return True

    @staticmethod
    def __determine_tolerance(reference,toleranceDivider):
        tolerance = 15 # matching all shapes with points within pixel range; was 15
        if (len(reference)>=3):
            smallestDistance = LineMatchClass.__getSmallestDistance(reference)
        else:
            smallestDistance = tolerance
        tolerance = smallestDistance / toleranceDivider
        if (tolerance<LineMatchClass.minTolerance):
            tolerance=LineMatchClass.minTolerance
        log.info('Adjusting tolerance to: %s' %tolerance)
        return tolerance

    '''
    Will not return matches with uneven lengths!
    toleranceDivider = 2 is suitable for lineMatch, maybe use different value for symbolposition
    '''
    def __init__(self,compvec,toleranceDivider = 2):
        # Note: We are caching reference here, because tolerance depends on it
        self.reference = compvec.copy()
        
        # prepare data, adjusting tolerance (only needed Once for input and reverse)
        # GET TOLERANCE FROM ONE
        # TOLERANCE NEEDS TO BE TAKEN FROM THE ONE THAT IS NOT GOING TO BE SCALED, preferably reference, because it can be predetermined -> DB/Cache
        self.tolerance = LineMatchClass.__determine_tolerance(self.reference, toleranceDivider)

    def scale(self,invec):
        # modification of comparison (and reference?) actually starts here

        #Scale comparison, not reference, because tolerance is already adjusted to reference
        # AND SCALE THE OTHER
        # EVERY __compare NEEDS A different SCALE (except forward/backward)
        invec = LineMatchClass.__scaleToAvg(invec,self.reference)
        return invec

    # this has only been separated from __init__ to allow overwriting
    def match(self,invec):
        comparison = invec.copy()

        # this is being done if not instance of SubDivision
        if not LineMatchClass.validate_data(comparison,self.reference):
            raise Exception('data validation failed')

        comparison = self.scale(comparison)

        log.debug('Comparing:')
        log.debug(comparison)
        log.debug('To:')
        log.debug(self.reference)

        # all we need from here is comparison
        log.info('Moving comparison to point 0,0')
        inAtZero = comparison.moveToZero()
        log.debug(inAtZero)

        forward = LineMatchClass.align(0,1,self.reference,inAtZero)

        if LineMatchClass.compare(self.tolerance,self.reference,forward):
            log.info('We have a match!')
            return True
        else:
            log.info('No match.')
            return False

if __name__ == "__main__":
    log = logSwitch.logPrint()
    log.info('Starting lineMatch...')
    input = array([[20,20],[55,130],[80,60],[120,85],[200,10]]).view(vectorTools.NumpyLine)
    log.info('rendering input.svg')
    #input.printVectors('input')
    dbrecord = array([[13,40],[50,125],[80,61],[123,79],[191,11]]).view(vectorTools.NumpyLine)
    log.info('rendering dbrecord.svg')
    #dbrecord.printVectors('dbrecord')
    matcher = LineMatchClass(dbrecord)
    print(type(matcher))
    print(dir(matcher))
    if matcher.match(input):
        print('Hooray!')



from PySide2.QtWidgets import QDockWidget,QTextEdit,QVBoxLayout,QGroupBox
from PySide2.QtCore import Qt, QSize
from stopeight.util.editor.runnable import EditorApp

import sys
import os
import io

class outwindow(QDockWidget):
    def __init__(self,parent=EditorApp().window):
        super(outwindow,self).__init__("LogWindow",parent)
        self.setMinimumSize(QSize(30, 0))

        self.text = QTextEdit()
        self.setWidget(self.text)
        self.text.setText("")
        self.data = None
        self.f = io.StringIO()

    #from contextlib import redirect_stdout
    #def __call__(self,data=None):
    #    with redirect_stdout(self.f)
    #        print('foobar')
    #        print(12)
    #        #libc.puts(b'this comes from C')
    #        os.system('echo and this is from echo')
    
    def update(self):
        self.text.setText(self.f.getvalue())
        self.show()
        self.f.close()
        self.f = io.StringIO()



# Copyright (C) 2009-2012 Specific Purpose Software GmbH

from operator import itemgetter
from numpy import ndarray,sqrt,square,zeros_like,pi
from numpy.core.umath import arctan2, arctan2
from stopeight.comparator.matrixTools import *

class NumpyLine(ndarray):
    def __new__(cls,*args,**kwargs):
        return ndarray.__new__(cls,*args,**kwargs)

    def reorderToNStart(self,n):
        reordered = zeros_like(self)
        for count in range(len(self)):
            if ((count+n)<len(self)):
                reordered[count]=self[count+n]
            else:
                reordered[count]=self[count-len(self)+n]
        return reordered

    def reverse(self):
        reversed = zeros_like(self)
        for pos,p in enumerate(self):
            reversed[len(self)-pos-1]=p
        return reversed

    def getAverageLength(self):
        sumDbLength = 0
        for dbCount,dbPoint in enumerate(self):
            if dbCount<len(self)-1:
                currPoint = dbCount+1
            elif dbCount==len(self)-1:
                currPoint = 0
            length = getVectorLength(self[currPoint]-dbPoint)
            sumDbLength += length
        avgDbLength = sumDbLength / len(self)
        return avgDbLength

    def moveToZero(self):
        movement = idmatrix()
        movement = dot(translate(-self[0]),movement)
        vAtZero = zeros_like(self)
        for pos,moveme in enumerate(self):
            vAtZero[pos] = transform(moveme,movement)
        return vAtZero

    def getAngleToNegativeXAxis(self):
        rad = arctan2( self[1], self[0]);
        deg = (rad/pi)*180.0 + 180.0;
        return deg

    def to_tuple(self):
        line = [(float(L[0]),float(L[1])) for L in self]
        return line

def getDistance(point1,point2):
    return getVectorLength([point1[0]-point2[0],point1[1]-point2[1]])

def getVectorLength(vector):
    return sqrt(square(vector[0]) + square(vector[1]))



#!/usr/bin/env python

# This code is from an example found on the web.
# only used for debugging math stuff.

# <!-- The quadratic Bezier curve -->
#     <path id = "quadcurveABC" d = "M 100 350 q 150 -300 300 0" stroke = "blue" stroke-width = "5" fill = "none"/>

import sys
import os
display_prog = '/usr/lib/x86_64-linux-gnu/qt5/examples/svg/svgviewer/svgviewer' # Command to execute to display images.

class Scene:
    def __init__(self,name="SVG.out",height=400,width=400):
        #width="744.09448"
        self.name = name
        self.items = []
        self.height = height
        self.width = width
        return

    def add(self,item): self.items.append(item)

    def strarray(self):
        var = ["<?xml version=\"1.0\"?>\n",
               "<svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 %d %d\" height=\"%d\" width=\"%d\">\n" % (self.width,self.height,self.height,self.width),
               " <g style=\"fill-opacity:1.0; stroke:black;\n",
               "  stroke-width:1;\">\n"]
        for item in self.items: var += item.strarray()
        var += [" </g>\n</svg>\n"]
        return var

    def set_height(self,height):
        self.height=height

    def write_svg(self,fileName=None):
        #if filename:
        #    self.svgname = filename
        #else:
        self.svgname = self.name + ".svg"
        file = open(self.svgname,'w')
        file.writelines(self.strarray())
        file.close()
        return

    def display(self,prog=display_prog):
        if os.system("%s %s &" % (prog,self.svgname)):
            print('Displaying %s' %self.svgname)
        else:
#            import importlib
#            if importlib.find_loader('SVGViewer') is not None:
            try:
                from stopeight.util import SVGViewer
                SVGViewer.show(self.svgname)
            except BaseException as e:
                print(e)
                print('SVGViewer not found')
        return


class Line:
    def __init__(self,start,end):
        self.start = start #xy tuple
        self.end = end     #xy tuple
        return

    def strarray(self):
        return ["  <line x1=\"%d\" y1=\"%d\" x2=\"%d\" y2=\"%d\" />\n" %\
                (self.start[0],self.start[1],self.end[0],self.end[1])]


class Circle:
    def __init__(self,center,radius,color):
        self.center = center #xy tuple
        self.radius = radius #xy tuple
        self.color = color   #rgb tuple in range(0,256)
        return

    def strarray(self):
        return ["  <circle cx=\"%d\" cy=\"%d\" r=\"%d\"\n" %\
                (self.center[0],self.center[1],self.radius),
                "    style=\"fill:%s;\"  />\n" % colorstr(self.color)]

class Rectangle:
    def __init__(self,origin,height,width,color):
        self.origin = origin
        self.height = height
        self.width = width
        self.color = color
        return

    def strarray(self):
        return ["  <rect x=\"%d\" y=\"%d\" height=\"%d\"\n" %\
                (self.origin[0],self.origin[1],self.height),
                "    width=\"%d\" style=\"fill:%s;\" />\n" %\
                (self.width,colorstr(self.color))]

class Quad:
    def __init__(self,start,mid,end):
        self.start = start
        self.mid = mid
        self.end = end

    def strarray(self):
        # start-x start-y q rel-mid-x 2x:rel-mid-y rel-end-x rel-end-y %d
        #return ["  <path d = \"M 70 230 q 130 -320 260 0\"  fill = \"none\"/>"]
        return ["  <path d = \"M %d %d Q %d %d %d %d\"  fill = \"none\"/>" % \
            (self.start[0],self.start[1],\
            (self.mid[0]),\
            (self.mid[1]),\
            (self.end[0]),\
            (self.end[1]))]

class Text:
    def __init__(self,origin,text,size=8):
        self.origin = origin
        self.text = text
        self.size = size
        return

    def strarray(self):
        return ["  <text x=\"%d\" y=\"%d\" font-size=\"%d\">\n" %\
                (self.origin[0],self.origin[1],self.size),
                "   %s\n" % self.text,
                "  </text>\n"]


def colorstr(rgb): return ("#%x%x%x" % (int(rgb[0]/16),int(rgb[1]/16),int(rgb[2]/16)))

def test():
    scene = Scene('SVG')
    scene.add(Rectangle((100,100),200,200,(0,255,255)))
    scene.add(Line((200,200),(200,300)))
    scene.add(Line((200,200),(300,200)))
    scene.add(Line((200,200),(100,200)))
    scene.add(Line((200,200),(200,100)))
    scene.add(Circle((200,200),30,(0,0,255)))
    scene.add(Circle((200,300),30,(0,255,0)))
    scene.add(Circle((300,200),30,(255,0,0)))
    scene.add(Circle((100,200),30,(255,255,0)))
    scene.add(Circle((200,100),30,(255,0,255)))
    scene.add(Text((50,50),"Testing SVG"))
    scene.add(Quad((70,230),(130,-320),(260,0)))
    scene.write_svg()
    scene.display()
    return

if __name__ == '__main__': test()



#!/usr/bin/env python

# Copyright (C) 2009-2012 Specific Purpose Software GmbH

from stopeight.comparator.matrixTools import *
from stopeight.comparator import vectorTools
from stopeight.comparator.shapeMatch import ShapeMatchClass

import stopeight.logging as log
log.disable(log.CRITICAL)

class ShapeMatchSubSectionClass:
    def __init__(self,compvec,toleranceDivider = 2):
        self.reference = compvec.copy()
        self.toleranceDivider = toleranceDivider

    def match(self,invec):
        '''
        This gives a boolean result for every line that is contained within another line.
        unequal length
        '''
        #multiple (instantiations) after change
        #order of vectors doesn't matter, will be flipped
        if (len(self.reference)<len(invec)):
            return False
            # under certain circumstances it may make sense to match a long input to a short reference
            #long = invec.copy()
            #short = self.reference.copy()
        else:
            short = invec.copy()
            long=self.reference.copy()            
        difference = len(long)-len(short)
        for count in range(difference+1):
            reordered = long.reorderToNStart(count)
            reordered = reordered[0:len(short)]
            unweightedReverse = ShapeMatchClass(short,self.toleranceDivider)
            if unweightedReverse.match(reordered):
                return True

if __name__ == "__main__":
    from stopeight.comparator import vectorTools
    log.info('Starting shapeMatch...')
    input = array([[20,20],[55,130],[80,60],[120,85],[200,10]]).view(vectorTools.NumpyLine)
    log.info('rendering input.svg')
    input.printVectors('input')
    dbrecord = array([[5,5],[13,40],[50,125],[80,61],[123,79],[191,11],[10,10]]).view(vectorTools.NumpyLine)
    log.info('rendering dbrecord.svg')
    dbrecord.printVectors('dbrecord')
    matcher = ShapeMatchSubSectionClass(dbrecord)
    if matcher.match(input):
        print('Hooray!')



# Copyright (C) 2018 Fassio Blatter

from stopeight.util.editor import runnable
from sys import modules as loader

from PySide2.QtWidgets import QComboBox,QPushButton

import stopeight.logging as log
_LOGDIR = '.stopeight' # this is not for logging messages; it is for data files

from inspect import signature
class Algorithm_Select(dict):
    def __init__(self, module, **kwargs):
        super(Algorithm_Select,self).__init__(**kwargs)
        self._module = module
        module_name = self._module[0]
        import importlib
        importlib.import_module(module_name)
        import types
        for key in dir(loader[module_name]):
            if not key.startswith('_'):
                builtin = isinstance(loader[module_name].__dict__[key],types.BuiltinFunctionType)
                dynamic = isinstance(loader[module_name].__dict__[key],types.FunctionType)
                if builtin or dynamic:
                    name = loader[module_name].__dict__[key].__name__
                    if builtin:
                        log.debug("Function "+name+" is C")
                    if dynamic:
                        log.debug("Function "+name+" is Python")
                    try:
                        return_type=signature(loader[module_name].__dict__[key]).return_annotation
                        try:
                            data_type=signature(loader[module_name].__dict__[key]).parameters['data'].annotation
                        except:
                            data_type = type(None)
                    except ValueError as v:
                        return_type = None
                        data_type = None
                    self[name]=(return_type,data_type)

import sys,traceback

class Algorithm_Run:

    @staticmethod
    def _auto_out(module_name,package_type,function_name):
        top = module_name+'.'+function_name
        import os
        sub = 'HEAD'
        import pkg_resources
        if (package_type==True):
            #get head from current directory (os.getcwd()).endswith('stopeight')
            if pkg_resources.require('stopeight')[0].version.find('+g'):
                sub=pkg_resources.require('stopeight')[0].version.split('+g')[1].split('.')[0]
            else:
                sub=pkg_resources.require('stopeight')[0].version
        else:
            #get head from stopeight-clibs
            sub=loader[module_name].version
        top = os.path.join(top,sub)
        return top

    @staticmethod
    def run(module,currentText,customsubpath,inputdata=None):
        import time
        time = time.time()
        outputdata=None
        try:
            if type(inputdata)!=type(None):
                try:
                    backup = inputdata[:]
                    log.info("Invoking "+currentText+" with "+str(len(inputdata))+" inputdata sets...")
                except TypeError:
                    #hack, should be copy of whole object
                    backup = inputdata.data[:]
                    log.info("Invoking "+currentText+" with custom input data type"+str(type(inputdata))+".")
                log.debug(str(inputdata))
                outputdata = loader[module[0]].__dict__[currentText](inputdata)
            else:
                backup = None
                log.info("Invoking "+currentText+" without inputdata.")
                outputdata = loader[module[0]].__dict__[currentText]()
                log.info("Size after call: Output "+str(len(outputdata)))

        #except:
        except AssertionError as ass:
            log.error(str(traceback.extract_tb(sys.exc_info()[2],limit=2).format()[1]))
        except BaseException as be:
            log.critical(str(sys.exc_info()[1])+" during method invokation: "+module[0]+'.'+currentText)
            from stopeight.util.editor.modules import file
            from os.path import expanduser,join
            file._write(backup,time,outdir=join(expanduser("~"),_LOGDIR
                                                ,Algorithm_Run._auto_out(module[0],module[1],currentText)
                                                ,customsubpath
                                                ))
            if outputdata!=None:
                try:
                    del outputdata[:]
                    if (len(outputdata)>0):
                        raise Exception("outputdata clear failed!")
                except TypeError:
                    pass
        return outputdata
    
from PySide2.QtCore import Qt
from contextlib import redirect_stdout,redirect_stderr
#def zoo(a: str)->int:
    #if (signature(zoo).return_annotation!=Signature.empty):
class Connector:
    def __init__(self,command,outputObjects,logwindow):
        self.logwindow = logwindow
        self._outputs = outputObjects
        self._module = command
        self.select = QComboBox()
        self.methods = Algorithm_Select(self._module)
        for method in iter(self.methods):
            self.select.addItem(method)
        self.button = QPushButton()
        self.button.setText("Run")
        self.button.clicked.connect(self.run)

    @staticmethod
    def _execute(executed,output,_module,functionName,customsubpath,logwindow,_data):
        if executed:
            raise Exception("There are multiple objects handling "+str(type(_data))+". The \
current version does not support handling multiple Input objects of the same type. Please remove "+str(type(_module))+" from module list.")
        with redirect_stderr(sys.stdout):
            with redirect_stdout(logwindow.f):
                try:
                    if type(_data) != type(None):
                        log.info("Executing "+functionName+" with "+str(type(_data)))
                        computed=Algorithm_Run.run(_module,functionName,customsubpath,inputdata=_data)
                    else:
                        log.info("Executing "+functionName+" with without data")
                        computed=Algorithm_Run.run(_module,functionName,"")
                    if output!=None:
                        if type(computed)!=type(output.data):
                            raise TypeError("functionreturn "+str(type(computed))+" of function "+str(functionName)+" not equal callannotation "+str(type(output.data)))
                        output(computed)
                        output.data=computed
                except AssertionError as ass:
                    log.info("AssertionError occurred during execution "+str(ass))
                except TypeError as tpe:
                    log.info("TypeError occurred during execution "+str(tpe))
                except BaseException as be:
                    raise be
        log.debug("update logwindow")
        logwindow.update()
        return True

    def run(self):
        log.setLevel(log.INFO)
        functionName = self.select.currentText()
        log.debug(str(self.methods))
        if functionName in self.methods:
            executed = False
            functionentry = (self.methods[functionName])[1]
            log.debug("Method "+str(functionName)+" has functionentry "+str(functionentry))
            functionreturn = (self.methods[functionName])[0]
            log.debug("Method "+str(functionName)+" has functionreturn "+str(functionreturn))
            #check return type of unique functionname
            if functionreturn==type(None):
                #custom types: check all outputs for matching type
                for _input in self._outputs:
                    inputtype = type(_input)
                    log.debug("Output has custom type "+str(inputtype))
                    if functionentry==inputtype:
                        log.debug("FunctionEntry "+str(functionentry)+" inputtype "+str(inputtype))
                        if not executed:
                            executed = Connector._execute(executed,None,self._module,functionName,str(inputtype.__name__),self.logwindow,_input)
            else:
                #generic types: check all outputs for receiving __call__
                for output in self._outputs:
                    callannotation = signature(output.__call__).parameters['data'].annotation
                    log.debug("Output has callannotation "+str(callannotation))
                    if functionreturn == callannotation:
                        # currently, only one input object is supported
                        # collecting data from all inputs
                        for _input in self._outputs:
                            log.debug("Type of input object data "+str(type(_input.data)))
                            if functionentry==type(_input.data):
                                executed = Connector._execute(executed,output,self._module,functionName,_input.identify(),self.logwindow,_input.data)
                        if not executed and functionentry==type(None):
                            executed = Connector._execute(executed,output,self._module,functionName,"",self.logwindow,None)
        else:
            raise NameError("Function not found")
        

#class WaveConnector(Connector):
#    def __init__(self,command,scribble):
#        super().__init__(command,[WaveData()],scribble)

#    def run(self):
#        super().run()



# Copyright (C) 2009-2012 Specific Purpose Software GmbH

from numpy import array,pi,cos,sin
#from SVG import Scene
#from numpy.oldnumeric.ma import dot
from numpy import dot

# This matrix is a: Column first style

_deg_to_rad = lambda deg: 2 * pi / (360.0/deg)

def idmatrix():
    return array([[1, 0, 0], [0, 1, 0], [0, 0, 1]])

def translate(a):
    m = array([[1,0,0],[0,1,0],[a[0],a[1],1]])
    return m

def scale(a):
    m = array([[a[0],0,0],[0,a[1],0],[0,0,1]])
    return m

def rotate(deg):
    if (deg!=0):
        c = cos(_deg_to_rad(deg))
        s = sin(_deg_to_rad(deg))
        m = array([[c,s,0],[-s,c,0],[0,0,1]])
    else:
        m = idmatrix()
    return m

def transform(a, m):
    b = array([0,0])
    b[0]= a[0] * m[0][0] + a[1]* m[1][0] + m[2][0]
    b[1]= a[0] * m[0][1] + a[1]* m[1][1] + m[2][1]
    return b

def center(a):
    return transform(a,translate(array([0,0])))

if __name__ == '__main__':
    a = array([30,30])
    m = idmatrix()
    print ('rotate vector with matrix:')
    m = dot(translate(array([10,15])),m)
    #m = dot(scale(array([0.5,0.5])),m)
    m = dot(rotate(-90),m)
    result = transform(a,m)
    print(a)
    print(result)
#    scene = Scene('matrixTools',255,255)
#    scene.add(Line(center(array([0,0])),center(a)))
#    scene.add(Line(center(array([0,0])),center(result)))
#    scene.write_svg()
#    scene.display()
#    return



# Copyright (C) 2018 Fassio Blatter
from stopeight import grapher
version=grapher.version

from stopeight.util.editor.data import ScribbleData, WaveData
from stopeight.util.editor.scribble import ScribbleArea

import stopeight.logging as log

def _append(data):
    for id,vector in enumerate(data):
        if id!=0:
            data[id]['coords']+=data[id-1]['coords']
    return data

def _extrema(data):
    from operator import itemgetter
    left,right,bottom,top = min(data,key=lambda k: k[0][0])[0][0],max(data,key=lambda k: k[0][0])[0][0],min(data,key=lambda k: k[0][1])[0][1],max(data,key=lambda k: k[0][1])[0][1]
    #if scribble
    bottom,top = top,bottom
    return left,right,bottom,top

def _scalingfactors(left,right,bottom,top,width,height):
    d_x = abs(right - left)
    d_y = abs(top - bottom)
    o_x = width
    o_y = height
    return o_x/d_x,o_y/d_y

def _resize(scribbledata,width,height):
    log.setLevel(log.INFO)
    assert type(scribbledata) is ScribbleData, "Wrong input data.data type: %r" % type(scribbledata)
    log.debug("Width "+str(width)+" Height "+str(height))
    from stopeight.matrix import Vectors,Stack
    vectors = Vectors(scribbledata)
    log.debug("First "+str(vectors.__array__()[0][0][0])+","+str(vectors.__array__()[0][0][1])+" Last "+str(vectors.__array__()[-1][0][0])+","+str(vectors.__array__()[-1][0][1]))
    stack=Stack()
    stack.identity()
    tx,ty=(-vectors.__array__()[0][0][0],-vectors.__array__()[0][0][1])
    log.debug("translating "+str(tx)+","+str(ty))
    stack.translate(tx,ty)
    vectors.apply(stack)
    log.debug("First "+str(vectors.__array__()[0][0][0])+","+str(vectors.__array__()[0][0][1])+" Last "+str(vectors.__array__()[-1][0][0])+","+str(vectors.__array__()[-1][0][1]))
    stack = Stack()
    stack.identity()
    from numpy.core.umath import arctan2
    from numpy import rad2deg
    angle = -rad2deg(arctan2(vectors.__array__()[-1][0][1],vectors.__array__()[-1][0][0]))
    log.debug("Rotating "+str(angle))
    stack.rotate(angle)
    vectors.apply(stack)
    log.debug("First "+str(vectors.__array__()[0][0][0])+","+str(vectors.__array__()[0][0][1])+" Last "+str(vectors.__array__()[-1][0][0])+","+str(vectors.__array__()[-1][0][1]))
    stack=Stack()
    stack.identity()
    left,right,bottom,top=_extrema(vectors.__array__())
    log.debug("Left "+str(left)+" Right "+str(right)+" Bottom "+str(bottom)+" Top "+str(top))
    log.debug("Width "+str(abs(right-left))+" Height "+str(abs(top-bottom)))
    sx,sy=_scalingfactors(left,right,bottom,top,width,height)
    log.info("Uniform scaling "+str(len(scribbledata))+" vectors with effective horiz: "+str(sx)+", and vert: "+str(sy)+" factors.")
    landscape= True if sx<sy else False
    log.debug("Mode landscape "+str(landscape))
    sx,sy= (sy,sy) if sx>sy else (sx,sx)
    log.debug("scaling "+str(sx)+","+str(sy))
    stack.scale(sx,sy)
    vectors.apply(stack)
    log.debug("First "+str(vectors.__array__()[0][0][0])+","+str(vectors.__array__()[0][0][1])+" Last "+str(vectors.__array__()[-1][0][0])+","+str(vectors.__array__()[-1][0][1]))
    left,right,bottom,top=_extrema(vectors.__array__())
    log.debug("Left "+str(left)+" Right "+str(right)+" Bottom "+str(bottom)+" Top "+str(top))
    stack=Stack()
    stack.identity()
    tx,ty=(-left,-top)
    log.debug("translating "+str(tx)+","+str(ty))
    stack.translate(tx,ty)
    vectors.apply(stack)
    log.debug("First "+str(vectors.__array__()[0][0][0])+","+str(vectors.__array__()[0][0][1])+" Last "+str(vectors.__array__()[-1][0][0])+","+str(vectors.__array__()[-1][0][1]))
    stack=Stack()
    stack.identity()
    tx,ty=((width-abs(right-left))/2,0) if not landscape else (0,(height-abs(top-bottom))/2)
    log.debug("translating "+str(tx)+","+str(ty))
    stack.translate(tx,ty)
    vectors.apply(stack)
    log.debug("First "+str(vectors.__array__()[0][0][0])+","+str(vectors.__array__()[0][0][1])+" Last "+str(vectors.__array__()[-1][0][0])+","+str(vectors.__array__()[-1][0][1]))
    testvec = vectors.__array__()
    log.debug("Rendering...")
    log.debug("First "+str(testvec[0][0][0])+","+str(testvec[0][0][1])+" Last "+str(testvec[-1][0][0])+","+str(testvec[-1][0][1]))
    return testvec.view(ScribbleData) 

def create_vector_graph(data:WaveData)->ScribbleData:
    assert type(data) is WaveData, "Input Error, wrong datatype: %r" % type(data)
    log.debug("Loading with "+str(len(data)))
    #from stopeight.matrix import Vectors #Hard crash if symbol used but missing: create_vector_graph return value
    from stopeight.grapher import VectorDouble,create_vector_graph
    import numpy as np
    invec = VectorDouble(data)
    #result = invec.create_vector_graph(1,1.0,True).__array__()
    result = create_vector_graph(invec,1,1.0,True)
    assert type(result) is np.ndarray, "Cast Error: %r" % type(result)
    result = _append(result)
    log.debug("Return Length "+str(len(result)))
    return result.view(ScribbleData)

#grapher data y inverted, scribble data y normal
def resize(data):
    log.setLevel(log.DEBUG)
    #doesnt work: sip.wrappertype
    #log.warning("data "+str(type(data)))
    #log.warning("ScribbleArea "+str(ScribbleArea.__class__))
    #assert data.__class__ is type(ScribbleArea), "Wrong input data type: %r" % data
    data(_resize(data.data,data.width(),data.height()))
    return None
resize.__annotations__ = {'data':ScribbleArea,'return':type(None)}



def test_Sequential():
    from stopeight.util.editor.modules import legacy
    from stopeight.util.parser import process_directory
    import os
    import stopeight.logging as log
    log.setLevel(log.CRITICAL)
    lines = process_directory(os.path.join('stopeight-clibs','legacy','test-data'),'.sp',legacy._parse_file,legacy.stroke_sequential,filename='Sequential.out',drawResize=False)

#def test_Parallel():
#    from stopeight.util.editor.modules import legacy
#    from stopeight.util.parser import process_directory
#    import os
#    import stopeight.logging as log
#    log.setLevel(log.CRITICAL)
#    lines = process_directory(os.path.join('stopeight-clibs','legacy','test-data'),'.sp',legacy._parse_file,legacy.stroke_parallel,filename='Parallel.out',drawResize=False)


#!/usr/bin/env python

# Copyright (C) 2017 Fassio Blatter
import stopeight.logging as log
log.basicConfig(level=log.DEBUG)

from stopeight.util.editor.runnable import EditorApp

from PySide2 import QtWidgets
from PySide2.QtWidgets import QToolBar,QGroupBox,QHBoxLayout

# False: Is part of a compiled library
_DATA = {'Modules': [
#                            ('stopeight.util.editor.modules.legacy', False),
                            ('stopeight.util.editor.modules.file', True),
                            ('stopeight.util.editor.modules.file_wave',True),
                            ('stopeight.util.editor.modules.grapher', False),
#                            ('stopeight.util.editor.modules.analyzer', False),
                    ]
         }

import importlib
active=[]
for module in _DATA['Modules']:
    try:
        importlib.import_module(module[0])
        active.append(module)
        log.warning("Successfully imported module "+module[0])
    except ImportError as ie:
        log.warning(ie)
        log.warning("Install stopeight-clibs or external library and reinstall stopeight afterwards to use module "+module[0])
    except:
        log.warning("Module "+module[0]+" not loaded!")
_DATA['Modules']=active
                
def main():
    log.setLevel(log.NOTSET)

    import sys
    app = EditorApp()

    connections = []

    outputs = []
    
    from stopeight.util.editor.wave import WaveArea
    wave = WaveArea()
    outputs.append(wave)
    
    wbar = QToolBar("WAV Viewer Toolbar")
    wgroup = QGroupBox()
    wbox = QtWidgets.QVBoxLayout()
    # Just some button connected to `plot` method
    plotbutton = QtWidgets.QPushButton('Plot')
    #from stopeight.util.editor.modules import file_wave
    #wave.data = file_wave._open()
    #plotbutton.clicked.connect(wave.plot)
    # set the layout
    wbox.addWidget(wave.plotbar)
    wbox.addWidget(plotbutton)
    wgroup.setLayout(wbox)
    wbar.addWidget(wgroup)
    EditorApp().window.addToolBar(wbar)

    EditorApp().window.setCentralWidget(wave.canvas)
    #EditorApp().window.addDockWidget(Qt.TopDockWidgetArea,wave)    

    # Create results area
    from stopeight.util.editor.scribble import ScribbleArea
    scribble = ScribbleArea()
    outputs.append(scribble)
    from PySide2.QtCore import Qt
    scribble.setAllowedAreas(Qt.AllDockWidgetAreas)
    EditorApp().window.addDockWidget(Qt.RightDockWidgetArea,scribble)

    # Find modules
    callables = []
    for module in _DATA['Modules']:
        callables.append(module)

    from stopeight.util.editor.callwindow import outwindow
    logwindow = outwindow()
    logwindow.setAllowedAreas(Qt.BottomDockWidgetArea)
    EditorApp().window.addDockWidget(Qt.BottomDockWidgetArea,logwindow)

    # Hook up modules
    toolbox = QToolBar("Algorithm Selector")
    #toolbox = QtWidgets.QDockWidget()
    from stopeight.util.editor.command import Connector
    for module in callables:
        connections.append(Connector(module,outputs,logwindow))
    for connection in connections:
        group = QGroupBox()
        box = QHBoxLayout()
        box.addWidget(connection.select)
        box.addWidget(connection.button)
        group.setLayout(box)
        toolbox.addWidget(group)
    EditorApp().window.addToolBar(Qt.BottomToolBarArea,toolbox)
    #EditorApp().window.addDockWidget(Qt.BottomDockWidgetArea,toolbox)
    
    EditorApp().window.show()
    sys.exit(EditorApp().exec_())

if __name__ == '__main__':
    main()



#!/usr/bin/env python

# Copyright (C) 2017 Fassio Blatter

import os
import numpy

import stopeight.logging as log
log.disable(log.WARNING)

def find_files(dir_path,suffix='.npy',base_dir=os.getcwd()):
    dir_path = os.path.join(base_dir,dir_path)
    paths = []
    if os.path.isdir(dir_path):
        log.debug('Parsing directory ' + dir_path)
        for root, dirs, files in os.walk(dir_path):
            for f in files:
                if f.endswith(suffix):
                    final_path = os.path.join(root,f)
                    paths.append(final_path)
    else:
        raise Exception('Path ' + dir_path + ' is not a directory under ' + os.getcwd())
    return paths

def process_directory(dir_path,suffix,file_adapter,analyzer,base_dir=os.getcwd(),filename:str=None,drawResize=True):
    if filename != None:
        from stopeight.util import tableau_printer
        tprinter = tableau_printer.tPrinter(filename,6)
    lines = []
    files = find_files(dir_path,suffix,base_dir)
    count = 0
    succeeded = 0
    for final_path in files:
        try:
            graph = file_adapter(final_path)
            log.info("SUCCESS: "+str(file_adapter.__name__) + " " + final_path)
            count+=1
            if filename != None:
                tprinter.draw(graph,drawResize)
            try:
                points = analyzer(graph)
                succeeded+=1
                if filename != None:
                    tprinter.draw(points,drawResize)
                try:
                    lines.append(points)
                except:
                    log.critical("Cast from Analyzer Failed")
                    return
            except:
                log.warning("FAILED: "+str(analyzer.__name__))
                if filename != None:
                    tprinter.text("#" + str(count) + " " + str(final_path))
        except:
            log.warning("FAILED: "+str(file_adapter.__name__) + " " + final_path)
    if filename != None:
        tprinter.text("Extracted " + str(len(lines)) + " Comparator lines out of " + str(count) + " readable Graph files.")
        tprinter.write()
    if len(lines)!=count:
        log.critical(str(len(files)-count) + " files out of " + str(len(files)) + " corrupted.")
    if count!=succeeded:
        log.critical(str(count-succeeded)+" inputs out of "+str(count)+" failed.")
    return lines



from stopeight.util import SVG
import os

import stopeight.logging as log
log.disable(log.CRITICAL)

class TCTPrinter:
    def __init__(self,filename):
        self.width=256
        self.height=256 #default, override before write
        self.scene = SVG.Scene(filename,self.height,self.width)
        self.offset = lambda xy: (xy[0],xy[1])
    
    def lines(self,nline):
        log.debug('Graphics adding to line queue '+str(nline))
        for i,l in enumerate(nline[0:-1]):
            log.debug('Graphics starting line from '+str(nline[i]))
            self.scene.add(SVG.Line((self.offset(nline[i])),(self.offset(nline[i+1]))))
            #self.scene.add(SVG.Line((200,200),(200,300)))

    def TCTs(self,TCTpath):
        log.debug('Segments adding to quad queue '+str(TCTpath))
        for quad in [TCTpath[x:x+3] for x in range(len(TCTpath)-2) if (x % 2 == 0)]:
            log.debug('Graphics starting quad '+str(quad))
            self.scene.add(SVG.Quad(quad[0],quad[1],quad[2]))

    def text(self,text):
        self.scene.add(SVG.Text(self.offset((0,128)),text))

    def write(self,display=False):
        self.scene.set_height(self.height)
        self.scene.write_svg()
        if display:
            self.scene.display()



from stopeight.util import SVG,TCT_printer
from stopeight.util.editor.modules.grapher import _resize
from stopeight.util.editor.data import ScribbleData

import stopeight.logging as log
log.disable(log.CRITICAL)

class tPrinter(TCT_printer.TCTPrinter):
    def __init__(self,filename,columns=2):
        TCT_printer.TCTPrinter.__init__(self,filename)
        self.counter=0
        self.columns=columns
        self.scene = SVG.Scene(filename,self.height,self.columns*self.width)
        offset_x = lambda x: x+((self.counter % self.columns) * self.width)
        offset_y = lambda y: y+((self.counter//self.columns) * self.height)
        self.offset = lambda xy: (offset_x(xy[0]),offset_y(xy[1]))
        self.scroll = lambda c: ((c//self.columns)+1) * self.height
    
    def draw(self,nline,resize=True):
        #assert type(nline) is type(ScribbleData)
        if resize:
            TCT_printer.TCTPrinter.lines(self,_resize(nline,256,256)['coords'])
        else:
            TCT_printer.TCTPrinter.lines(self,nline['coords'])
        self.counter+=1

    def text(self,text):
        TCT_printer.TCTPrinter.text(self,text)
        self.counter+=1

    def write(self):
        self.scene.set_height(self.scroll(self.counter))
        self.scene.write_svg()